﻿using System;

namespace BaseConcepts_03
{
    public interface IVehicle
    {
        void Accelerate();
    }

    public interface ICargoVehicle : IVehicle
    {
        int GoodsCapacity { get; }
    }

    public interface IPersonalVehicle : IVehicle
    {
        int PassengerCapacity { get; set; }
    }

    public interface IHeavyWeightVehicle
    {
        float MaximalLoadInKg { get; }
    }

    public class Truck : ICargoVehicle, IHeavyWeightVehicle
    {
        public int GoodsCapacity { get; }
        // interface declares only get method, but we could also declare a private setter
        public float MaximalLoadInKg { get; }


        public Truck(int goodsCapacity, float maxLoad)
        {
            GoodsCapacity = goodsCapacity;
            MaximalLoadInKg = maxLoad;
        }

        public void Accelerate()
        {
            Console.WriteLine("Accelerating slowly...");
        }
    }

    public class Car : IPersonalVehicle
    {
        // if property had initializer, it would be ignored
        public int PassengerCapacity { get; set; }

        // CTOR with default value
        public Car(int capacity = 0)
        {
            // 5 is the default car capacity
            PassengerCapacity = capacity > 0 ? capacity : 5;
        }

        public void Accelerate()
        {
            Console.WriteLine("Accelerating fast...");
        }
    }
}